import copy
import types
import matplotlib.pyplot as plt
import random as rnd
import collections.abc

def make1d(n, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make1d: first argument should be an integer")
    if random:
        return [rnd.random() for i in range(0,n)]
    else:
        return [copy.deepcopy(value) for i in range(0,n)]

def make2d(n, m, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make2d: first argument should be an integer")
    if not type(m) is int:
        raise TypeError("make2d: second argument should be an integer")
    return [make1d(m, value=value, random=random) for i in range(0,n)]

def make3d(n, m, k, value=0, random=False):
    if not type(n) is int:
        raise TypeError("make3d: first argument should be an integer")
    if not type(m) is int:
        raise TypeError("make3d: second argument should be an integer")
    if not type(k) is int:
        raise TypeError("make3d: third argument should be an integer")
    return [make2d(m, k, value=value, random=random) for i in range(0,n)]

def print2d(data, rowLabels=None, colLabels=None): #not used in the textbook
    if not (isinstance(data, collections.abc.Sequence) and
            isinstance(data[0], collections.abc.Sequence)):
        raise TypeError("print2d: argument should be a 2d-array")
    tbl = plt.table(cellText=[[str(j) for j in i] for i in data],
                    bbox=[0,0,1,1], cellLoc='center', loc='top',
                    rowLabels=rowLabels, colLabels=colLabels)
    tbl.auto_set_font_size(False)
    tbl.set_fontsize(16)
    plt.axis('off')
    plt.show()



import numpy
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.image as image
import matplotlib.animation as animation
import copy
import collections.abc

if 'jshtml' in matplotlib.rcsetup.validate_movie_html_fmt.valid:
  matplotlib.rc('animation', html='jshtml')
else:
  matplotlib.rc('animation', html='html5')

def histogram_plot(*args, **kwargs): #not used in the textbook
    plt.hist(*args, **kwargs)
    plt.show()

def image_preprocess(data):
  for i in range(0,len(data)):
    for j in range(0,len(data[i])):
      if isinstance(data[i][j], collections.abc.Sequence):
        if len(data[i][j]) < 3:
            raise ValueError("color pixel should be an array of length 3")
        for k in range(0,3):
          data[i][j][k] = float(data[i][j][k])
      else:
        if not (type(data[i][j]) is int or type(data[i][j]) is float):
          raise TypeError("monochrome image should be a 2d-array of numbers")
        v = float(data[i][j])
        data[i][j] = [v, v, v]

def image_show(data):
    if not (isinstance(data, collections.abc.Sequence) and
            isinstance(data[0], collections.abc.Sequence)):
        raise TypeError("image_show: argument should be an image (i.e., a 2d-array)")
    tmp = copy.deepcopy(data)
    image_preprocess(tmp)
    picture = plt.imshow(tmp)
    plt.show()
    return picture

def animation_show(data):
    if not (isinstance(data, collections.abc.Sequence) and
            isinstance(data[0], collections.abc.Sequence) and
            isinstance(data[0][0], collections.abc.Sequence)):
        raise TypeError("animation_show: argument should be an array of images (i.e., a 2d-array)")
    fig = plt.figure()
    a = []
    for i in range(0,len(data)):
        tmp = copy.deepcopy(data[i])
        image_preprocess(tmp)
        a.append([plt.imshow(tmp)])
    ani = animation.ArtistAnimation(fig, a, interval=500)
    plt.show()
    return ani

def linear_fit(data):
  if not isinstance(data, collections.abc.Sequence):
    TypeError("linear_fit: argument should be an array")
  xdata = [i[0] for i in data]
  ydata = [i[1] for i in data]
  fitp = numpy.poly1d(numpy.polyfit(xdata, ydata, 1))
  ma = max(xdata)
  mi = min(xdata)
  xp = numpy.linspace(mi, ma, (ma - mi))
  plt.plot(xdata, ydata, '.', xp, fitp(xp), '-')
  plt.show()

import matplotlib.cm as cm

def plotdata(data, line=False, **kwargs):
  if line:
    pltfun = plt.plot
  else:
    pltfun = plt.scatter
  if not isinstance(data, collections.abc.Sequence):
    TypeError("plotdata: argument should be an array")
  if isinstance(data[0], collections.abc.Sequence):
    if isinstance(data[0][0], collections.abc.Sequence):
      plot_clusters(data)
    else:
      xdata = [ i[0] for i in data ]
      ydata = [ i[1] for i in data ]
      pltfun(xdata, ydata, **kwargs)
  else:
    xdata = [ i for i in range(0,len(data)) ]
    pltfun(xdata, data, **kwargs)
  plt.show()

def plot_clusters(data, **kwargs):
  cnum = len(data)
  n = 0
  for x in data:
    n = n + len(x)
  xdata = [0] * n
  ydata = [0] * n
  colors = [0.0] * n
  k = 0
  for i in range(0,len(data)):
    for j in data[i]:
      xdata[k] = j[0]
      ydata[k] = j[1]
      colors[k] = cm.hsv(i/cnum)
      k = k + 1
  plt.scatter(xdata, ydata, c = colors, **kwargs)
  plt.show()





from matplotlib import pyplot
import time
import types
import collections

def argToStr(arg):
    if isinstance(arg, collections.abc.Sequence) and len(arg) > 3:
        return "[" + argToStr(arg[0]) + "," + argToStr(arg[1]) + ", ...]"
    else:
        return str(arg)

def evalWithTime(f, arg, c):
    s = f.__name__ + "(" + argToStr(arg) + ")"
    print("evaluating ", s, "... ", end="")
    start = time.time()
    for i in range(c):
        if isinstance(arg, tuple):
            f(*arg)
        else: 
            f(arg)
    et = (time.time() - start) / c
    print("finished in ", et, " seconds.")
    return et

def bench(f, args, measure=(lambda x: x), count=1):

    if not hasattr(f, '__call__'):
        raise TypeError("bench: first argument should be a function")
    if not isinstance(args, collections.abc.Sequence):
        raise TypeError("bench: second argument should be a series of arguments")
    x = [measure(i) for i in args]
    y = [evalWithTime(f,i,count) for i in args]
    return (x,y)

def plot(d, xlogscale=False, ylogscale=False):
    if xlogscale:
        pyplot.xscale("log")
    else:
        pyplot.xscale("linear")
    if ylogscale:
        pyplot.yscale("log")
    else:
        pyplot.yscale("linear")
    pyplot.plot(d[0],d[1])
    pyplot.show()




from setuptools import setup
from setuptools import find_packages
from codecs import open

with open('README', encoding='utf-8') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='ita',
    version='0.0.14',
    description='support package for "introduction to algorithm" lecture',
    long_description=readme,
    author='Akimasa Morihata',
    author_email='morihata@graco.c.u-tokyo.ac.jp',
    url='https://lecture.ecc.u-tokyo.ac.jp/johzu/joho-kagaku/',
    license=license,
    install_requires=['numpy', 'matplotlib' ],
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent" ]
)
